// -------------------------------------------------------------------------------- 
//!  @file  main.c                                                                  
//!  メイン処理のソースファイル                                                     
//                                                                                  
//   Copyright (C) 2021 lsluk, All rights reserved.                                 
//   http://vdmgr.g2.xrea.com/index.html                                            
// -------------------------------------------------------------------------------- 
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <pico/stdlib.h>
#include "config.h"
#include "libvga/vga/vga.h"
#include "libvga/pad/pad.h"
#include "libvga/dac/mcp49x1.h"
#include "libvga/pcm/ay-3-8910.h"
#include "libvga/pcm/scc.h"

#define AY38910_SCC												// 定義時は AY-3-8910 + SCC

#ifndef AY38910_SCC
#include "sounddata0.c"
#else
#include "sounddata1.c"
#endif//AY38910_SCC

#ifdef  SOUNDDAC_CFG_DAC_MCP49x1
#define DAC_INIT					mcp49x1_init					//!< DAC 初期化処理
#define DAC_WRITE					mcp49x1_write					//!< DAC 書き込み処理
#define DAC_WAIT					mcp49x1_wait					//!< DAC 書き込み待ち処理
#endif//SOUNDDAC_CFG_DAC_MCP49x1


static struct repeating_timer		m_timer ;						//!< 繰り返しタイマー
static PCMAY38910INFO				m_ay38910Info ;					//!<
static PCMSCCINFO					m_sccInfo ;						//!< SCC 情報
static uint16_t						m_output ;						//!< 出力値
static uint32_t						const *m_p_soundData ;			//!< サウンドデータ


// -------------------------------------------------------------------------------- 
//!  繰り返しタイマー割り込みサービスルーチン                                       
//!                                                                                 
//!  @param [in] p_timer  繰り返しタイマー                                          
//!  @retval     true     繰り返し                                                  
//!  @retval     false    停止                                                      
// -------------------------------------------------------------------------------- 
static bool RAMFUNC isr_timer( struct repeating_timer *p_timer )
{
	int32_t		output ;


	mcp49x1_write( m_output ) ;

	output   = pcm_ay_3_8910_update( &m_ay38910Info ) ;
	output  += pcm_scc_update( &m_sccInfo ) ;

	m_output = (uint16_t )(output + 0x0800) ;

	mcp49x1_wait() ;

	return true ;
}


// -------------------------------------------------------------------------------- 
//!  VSync 処理                                                                     
//!                                                                                 
//!  @return なし                                                                   
// -------------------------------------------------------------------------------- 
static void RAMFUNC isr_vsync(uint gpio, uint32_t events)
{
	//
	pcm_ay_3_8910_setChannelReg(&m_ay38910Info, 0, m_p_soundData[0]) ;
	pcm_ay_3_8910_setChannelReg(&m_ay38910Info, 1, m_p_soundData[1]) ;
	pcm_ay_3_8910_setChannelReg(&m_ay38910Info, 2, m_p_soundData[2]) ;

	if( m_p_soundData[3] & 0x80000000 ) {
		pcm_ay_3_8910_setNoiseEnvelopeReg(&m_ay38910Info, m_p_soundData[3]) ;
	} else {
		pcm_ay_3_8910_setEnvelopePeriod(&m_ay38910Info, (uint16_t )m_p_soundData[3]) ;
	}

	m_p_soundData += 4 ;

#ifdef  AY38910_SCC
{
	uint16_t	const *p_sccData ;

	//
	p_sccData      = (uint16_t const *)&m_p_soundData[0] ;
	m_p_soundData += 2 ;

//#	pcm_scc_setReg(&m_sccInfo, 0, m_p_soundDataSCC[0]) ;
	pcm_scc_setReg(&m_sccInfo, 1, p_sccData[0]) ;
	pcm_scc_setReg(&m_sccInfo, 2, p_sccData[1]) ;
	pcm_scc_setReg(&m_sccInfo, 3, p_sccData[2]) ;
	pcm_scc_setReg(&m_sccInfo, 4, p_sccData[3]) ;
}
#endif//AY38910_SCC

	if( m_p_soundData >= &m_soundData[sizeof( m_soundData )/4] ) {
#ifndef  AY38910_SCC
		m_p_soundData = &m_soundData[0] ;
#else
		m_p_soundData = &m_soundData[6*565] ;
#endif//AY38910_SCC
	}
}


// -------------------------------------------------------------------------------- 
//!  初期化                                                                         
//!                                                                                 
//!  @return  なし                                                                  
// -------------------------------------------------------------------------------- 
void sound_init( void )
{
	int		index ;


	DAC_INIT() ;

	m_output      = 0x0800 ;
	m_p_soundData = &m_soundData[0] ;

	pcm_ay_3_8910_reset( &m_ay38910Info ) ;
	pcm_scc_reset( &m_sccInfo ) ;

#ifdef  AY38910_SCC
	pcm_scc_copyWaveData(&m_sccInfo, 0, (int8_t const *)&m_waveData[0], 128) ;
	pcm_scc_enable(&m_sccInfo, 0x1f) ;
#endif//AY38910_SCC

	for(index=0; index<=0x0800; index+=0x0040) {
		DAC_WRITE( index ) ;
		DAC_WAIT() ;
		sleep_us( 10 ) ;
	}

	add_repeating_timer_us(-SOUNDDAC_CFG_TIME, isr_timer, NULL, &m_timer) ;

	gpio_set_irq_enabled_with_callback(VGA_CFG_PIN_VSYNC, GPIO_IRQ_EDGE_FALL, true, isr_vsync) ;
}


// -------------------------------------------------------------------------------- 
//!  メイン処理                                                                     
//!                                                                                 
//!  @param [in] argc  引数の数                                                     
//!  @param [in] argv  引数                                                         
//!  @return           常に０                                                       
// -------------------------------------------------------------------------------- 
int RAMFUNC main(int argc, char *argv[])
{
	setup_default_uart() ;

	sound_init() ;

	vga_init() ;
	vga_start() ;

	while( 1 ) {
		;
	}

	return 0 ;
}
